import linecache
import sys
import traceback

def printException():
#     exc_type, exc_obj, tb = sys.exc_info()
#     f = tb.tb_frame
#     lineno = tb.tb_lineno
#     filename = f.f_code.co_filename
#     linecache.checkcache(filename)
#     line = linecache.getline(filename, lineno, f.f_globals)
#     print 'EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj)
    traceback.print_exc(file=sys.stdout)


from setuptools import setup


import urllib, zipfile, os
from distutils.sysconfig import get_python_lib
def get_CGEA_data():
    package_path = get_python_lib()
    urllib.urlretrieve("https://bitbucket.org/MaxTomlinson/cgea/downloads/data.zip", 
                       package_path+"/CGEA/data.zip")
    zipfile.extractall(package_path+"/CGEA/data.zip")
    os.remove(package_path+"/CGEA/data.zip")

setup(name='CGEA',
      version='0.9',
      description='Chemo Genomic Enrichment Analysis',
      url='https://bitbucket.org/MaxTomlinson/cgea',
      author='Max Tomlinson',
      author_email='max.tomlinson@mssm.edu',
      license='MIT',
      packages=['CGEA'],
      install_requires=[
          'jsonpickle',
          'numpy',
          'scikit-learn',
          'scipy',
          'statsmodels',
          'pandas'
      ],
      cmdclass = {'my_command': get_CGEA_data},
      include_package_data=True,
      zip_safe=False)





from featureBasedCGEA import CGEAdata
from statistics import makeMixmdl


data = CGEAdata()
cmap = data.get('cmap')
unifiedCompoundLevelSets = data.get("unifiedCompoundLevelSets")
mixmdls = data.getModel('mixmdl')
setSizes = reduce(lambda x,y: set.union(x,set(map(len,y.values()))), unifiedCompoundLevelSets.values(), set())
makeMixmdl(cmap.keys(), data.getPath('mixmdl'), setSizes = setSizes, numIterations = 100000)


import collections
import itertools
import multiprocessing

class SimpleMapReduce(object):
    
    def __init__(self, map_func, reduce_func, num_workers=None):
        """
        map_func

          Function to map inputs to intermediate data. Takes as
          argument one input value and returns a tuple with the key
          and a value to be reduced.
        
        reduce_func

          Function to reduce partitioned version of intermediate data
          to final output. Takes as argument a key as produced by
          map_func and a sequence of the values associated with that
          key.
         
        num_workers

          The number of workers to create in the pool. Defaults to the
          number of CPUs available on the current host.
        """
        self.map_func = map_func
        self.reduce_func = reduce_func
        self.pool = multiprocessing.Pool(num_workers)
    
    def partition(self, mapped_values):
        """Organize the mapped values by their key.
        Returns an unsorted sequence of tuples with a key and a sequence of values.
        """
        partitioned_data = collections.defaultdict(list)
        for key, value in mapped_values:
            partitioned_data[key].append(value)
        print partitioned_data
        return partitioned_data.items()
    
    def __call__(self, inputs, chunksize=1):
        """Process the inputs through the map and reduce functions given.
        
        inputs
          An iterable containing the input data to be processed.
        
        chunksize=1
          The portion of the input data to hand to each worker.  This
          can be used to tune performance during the mapping phase.
        """
        map_responses = self.pool.map(self.map_func, inputs, chunksize=chunksize)
        if isinstance(map_responses, dict):
            partitioned_data = self.partition(itertools.chain(*map_responses))
            reduced_values = self.pool.map(self.reduce_func, partitioned_data)
        else:
            reduced_values = self.reduce_func(map_responses)
        return reduced_values